@@ -3,5 +3,5 @@
 Python wrapper for the official Bundestag API
 """
 
-from .bta_wrapper import BTA_Connection, Person, Role, Drucksache, \
+from .bta_wrapper import btaConnection, Person, Role, Drucksache, \
     Aktivitaet, Vorgang, Vorgangsposition, Plenarprotokoll
